@@ -1,9 +1,11 @@
-# https://github.com/alisoltanirad/Logic.git
-
 class Sentence:
 
     def __invert__(self):
-        return Invert(self)
+        if type(self) is Invert:
+            return self.child
+        else:
+            return Invert(self)
+
 
     def __and__(self, other):
         return Conjunction(self, other)
@@ -58,7 +60,10 @@ class Invert(Sentence):
         return hash(not self.child)
 
     def validate(self, set):
-        return (not self.child) in set
+        if type(self.child) is Atomic:
+            return not self.child in set
+        else:
+            return not self.child.validate(set)
 
 
 class Conjunction(BinarySentence):
@@ -88,7 +93,7 @@ class Implication(BinarySentence):
         return hash(not self.lchild or self.rchild)
 
     def validate(self, set):
-        return not self.lchild.validate(set) or self.rchild.validate(set)
+        return (~self.lchild.validate(set)) or self.rchild.validate(set)
 
 
 class Equality(BinarySentence):
@@ -100,7 +105,7 @@ class Equality(BinarySentence):
 
     def validate(self, set):
         return (
-            (self.lchild.validate(set)) == True and (self.rchild.validate(set)) == True
+            self.lchild.validate(set) and self.rchild.validate(set)
         ) or (
-            ((~self.lchild).validate(set)) == True and ((~self.rchild).validate(set)) == True
+            (~self.lchild).validate(set) and (~self.rchild).validate(set)
         )
